import argparse
import networkx as nx
import pandas as pd
import subprocess

#import markov_clustering as mc
#parser = argparse.ArgumentParser()
#parser.add_argument("edges",type=str)
#parser.add_argument("out",type=str)
#parser.add_argument("inflation",type=str)
#args = parser.parse_args()


def run_mcl(args):
	edgedf = pd.read_csv(args.edges,sep='\t',header=None)
	edgedf.columns = ['node1','node2','weight']
	edgeset = set()
	distg = nx.Graph()

	outfile = open('tmp/input2mcl','w')

	for n1,n2,w in zip(edgedf['node1'],edgedf['node2'],edgedf['weight']):
		key1 = n1+'#'+n2
		key2 = n2+'#'+n1
		if key1 in edgeset or key2 in edgeset:
			continue
		edgeset.add(key1)
		outfile.write(n1+'\t'+n2+'\t'+str(w)+'\n')

		distg.add_edge(n1,n2,weight=w)
		
	nodes = list(distg.nodes())
#mtx = nx.to_scipy_sparse_array(distg)
	outfile.close()

	inflation = args.inflation
	subprocess.run(['mcl', args.edges,'--abc','-I', str(inflation),'-o','mclout'])
	'''
	result = mc.run_mcl(mtx,inflation=inflation)
	clusters = mc.get_clusters(result)
	'''
	mclout = open('mclout','r')
	
	data = []
	for line in mclout:
		cl = line.strip().split('\t')
		if len(cl) < 2:
			continue
		members = []
		for member in cl:
			
			members.append(member)
		memberstr = ','.join(members)
		data.append([len(members),memberstr])
	mclout.close()
	df = pd.DataFrame(data,columns=['Size','Members'])
	df.to_csv('tmp/mcl'+str(inflation))



import subprocess
import os



#util code adopted from vcontact2
def make_diamond_db(aa_fp, db_dir, cpu: int):

    diamond_db_bp = os.path.join(db_dir, os.path.basename(aa_fp).rsplit('.', 1)[0])
    make_diamond_cmd = ['diamond', 'makedb', '--threads', str(cpu), '--in', aa_fp, '-d', diamond_db_bp]


    res = subprocess.run(make_diamond_cmd, check=True, stdout=subprocess.PIPE)

    diamond_db_fp = diamond_db_bp + '.dmnd'

    return diamond_db_fp


def run_diamond(aa_fp, db_fp, cpu: int, evalue: float, alignments: int, diamond_out_fn):

    # More sensitive as an option?
    diamond_cmd = ['diamond', 'blastp', '--threads', str(cpu),
                   '--sensitive', '--evalue', str(evalue), '--max-target-seqs', str(alignments),
                   '-d', db_fp, '-q', aa_fp, '-o', diamond_out_fn]


    res = subprocess.run(diamond_cmd, check=True, stdout=subprocess.PIPE)


    return diamond_out_fn


def make_protein_clusters_mcl(blast_fp, out_p, inflation=2):
    """
    Args: 
        blast_fp (str): Path to blast results file
        inflation (float): MCL inflation value
        out_p (str): Output directory path
    Returns:
        str: fp for MCL clustering file
    """


    blast_fn = os.path.basename(blast_fp)
    abc_fn = '{}.abc'.format(blast_fn)
    abc_fp = os.path.join(out_p, abc_fn)
    subprocess.check_call("awk '$1!=$2 {{print $1,$2,$11}}' {0} > {1}".format(blast_fp, abc_fp), shell=True)



    mci_fn = '{}.mci'.format(blast_fn)
    mci_fp = os.path.join(out_p, mci_fn)
    mcxload_fn = '{}_mcxload.tab'.format(blast_fn)
    mcxload_fp = os.path.join(out_p, mcxload_fn)
    subprocess.check_call("mcxload -abc {0} --stream-mirror --stream-neg-log10 -stream-tf 'ceil(200)' -o {1}"
                          " -write-tab {2}".format(abc_fp, mci_fp, mcxload_fp), shell=True)

    mcl_clstr_fn = "{0}_mcl{1}.clusters".format(blast_fn, int(inflation*10))
    mcl_clstr_fp = os.path.join(out_p, mcl_clstr_fn)

    subprocess.check_call("mcl {0} -I {1} -use-tab {2} -o {3}".format(
        mci_fp, inflation, mcxload_fp, mcl_clstr_fp), shell=True)

    return mcl_clstr_fp



from sklearn.cluster import AffinityPropagation
from sklearn.cluster import *
import pandas as pd
import numpy as np
from apclusterv.cluster import adj_ap
import networkx as nx
import json
import argparse

#parser = argparse.ArgumentParser()
#parser.add_argument('c1abs')
#parser.add_argument('cluster')
#usage: runcluster(args)

var2value = {}
var2value['k'] = 10




complements = []
c2size = {}
c2sim = {}
c2members = {}


def sumcluster(cluster):
	cldf = pd.read_csv(cluster,sep=',',header=0)

	rep2ctg = {}
	ctg2rep = {}
	for idx,row in cldf.iterrows():
		ctg2rep[row['contig']] = row['rep']
		if row['rep'] in rep2ctg.keys():
			rep2ctg[row['rep']].append(row['contig'])
		else:
			rep2ctg[row['rep']] = [row['contig']]
	print(len(rep2ctg))
	return rep2ctg,ctg2rep

def separate(rep2ctg,ctg2rep,c1abs):
	rep2edges = {}
	for rep in rep2ctg.keys():
		rep2edges[rep] = []

	edge1 = pd.read_csv(c1abs,sep=',',header=0)
	
	

	for ctg1,ctg2,score in zip(edge1['ctg1'],edge1['ctg2'],edge1['ratioscore']):
		if not ctg1 in ctg2rep.keys():
			continue
		if not ctg2 in ctg2rep.keys():
			continue

		if ctg2rep[ctg1] != ctg2rep[ctg2]:
			continue
		
		rep2edges[ctg2rep[ctg1]].append([ctg1,ctg2,score])

	
	
	return rep2edges

def printresult():

	with open('tmp/c2members','w') as f1:
		json.dump(c2members,f1)
	with open('tmp/c2size','w') as f2:
		json.dump(c2size,f2)
	with open('tmp/c2sim','w') as f3:
		json.dump(c2sim,f3)

	
def runcluster(args):
	var2value['k'] = args.apk
	rep2ctg,ctg2rep = sumcluster(args.repfile)
	rep2edges = separate(rep2ctg,ctg2rep,args.edgefile)

	for rep,edges in rep2edges.items():

		
		ctg2id,id2ctg = construct(edges)
		if len(ctg2id) < 5:
			continue
		#outfile = open('sres/{}.res'.format(rep),'w')
		aptest(edges,ctg2id,id2ctg)
		#outfile.close()
	printresult()
	

def construct(edges):
	ctg2id = {}
	id2ctg = {}

	for ctg1,ctg2,score in edges:
		if not ctg1 in ctg2id.keys():
			thisid = len(ctg2id)
			ctg2id[ctg1] = thisid
			id2ctg[thisid] = ctg1
			
		if not ctg2 in ctg2id.keys():
			thisid = len(ctg2id)
			ctg2id[ctg2] = thisid
			id2ctg[thisid] = ctg2 
	return ctg2id,id2ctg
'''
def construct(crossgenus,ingenus,rep2edges):


	cldf = pd.read_csv(cluster,sep='\t',header=None)
	cldf.columns = ['i','rep','contig']
	ctgset = set(cldf['contig'])
	edges = []
	edge1 = pd.read_csv(crossgenus,sep='\t',header=None)
	edge1.columns = ['ctg1','ctg2','e','score','count','genus1','genus2']
	
	for ctg1,ctg2,e,score in zip(edge1['ctg1'],edge1['ctg2'],edge1['e'],edge1['score']):
		if not ctg1 in ctgset:
			continue
		if not ctg2 in ctgset:
			continue

		if not ctg1 in ctg2id.keys():
			thisid = len(ctg2id)
			ctg2id[ctg1] = thisid
			id2ctg[thisid] = ctg1
			
		if not ctg2 in ctg2id.keys():
			thisid = len(ctg2id)
			ctg2id[ctg2] = thisid
			id2ctg[thisid] = ctg2 
		
		edges.append([ctg1,ctg2,e,score])
	
	edge2 = pd.read_csv(ingenus,sep='\t',header=None)
	edge2.columns = ['ctg1','ctg2','e','score','count','genus1','genus2']
	for ctg1,ctg2,e,score in zip(edge2['ctg1'],edge2['ctg2'],edge2['e'],edge2['score']):
		if not ctg1 in ctgset:
			continue
		if not ctg2 in ctgset:
			continue

		if not ctg1 in ctg2id.keys():
			thisid = len(ctg2id)
			ctg2id[ctg1] = thisid
			id2ctg[thisid] = ctg1
			
		if not ctg2 in ctg2id.keys():
			thisid = len(ctg2id)
			ctg2id[ctg2] = thisid
			id2ctg[thisid] = ctg2 
			
		edges.append([ctg1,ctg2,e,score])
	#print(edges)    
	return edges
'''
def aptest(edges,ctg2id,id2ctg):
	
	N = len(id2ctg)
	mtx = np.zeros((N,N))

	simg =  nx.Graph()
	for ctg1,ctg2,score in edges:
		ctgid1 = ctg2id[ctg1]
		ctgid2 = ctg2id[ctg2]
		
		sim = score
   
		#simg.add_edge(ctg1,ctg2,weight=sim)
		mtx[ctgid1,ctgid2] = sim
		mtx[ctgid2,ctgid1] = sim
	#print(mtx,mtx.shape,np.max(mtx),np.mean(mtx))
	#mtx = mtx/np.max(mtx)
	for i in range(N-1):
		for j in range(i+1,N):
			if mtx[i,j] == 0:
				continue
			simg.add_edge(i,j,weight=mtx[i,j])
		

	#print(simg.edges(data=True))
	clcoef = nx.clustering(simg,weight='weight')
	between = nx.betweenness_centrality(simg)
	
	preference = np.zeros(N)
	#preference = np.median(mtx)
	#print(clcoef)
	pstep = np.zeros(N)


	for node,coef in clcoef.items():
		#print(node,coef,between[node])
		print('k',var2value['k'])
		#preference[node] = coef - var2value['k']*between[node]
		preference[node] = coef
		pstep[node] = 0.01
	preference = np.clip(preference,0,1)
	#preference = np.mean(mtx,axis=0)
	#pstep = (np.max(mtx) - np.min(mtx) )/100
	#print(ctg2id)
	#cluster_centers_indices, labels = affinity_propagation(mtx, max_iter=1000,preference=preference, random_state=39)
	print(preference)
	cluster_centers_indices, labels = adj_ap.adj_affinity_propagation(mtx,damping=0.2,pstep=pstep, max_iter=2000,preference=preference, random_state=39)
	centerstr  = ''

	print(cluster_centers_indices)
	for center in cluster_centers_indices:
		centerstr+=str(id2ctg[center])+','
		complements.append(id2ctg[center])
		c2size[id2ctg[center]] = 0
		c2sim[id2ctg[center]] = -1
		c2members[id2ctg[center]] = []

	idx = 0
	for label in labels:
		examplar = id2ctg[cluster_centers_indices[label]]
		ctg = id2ctg[idx]
		c2members[examplar].append(ctg)
		if cluster_centers_indices[label] != idx:
			c2sim[examplar] = max(c2sim[examplar],mtx[cluster_centers_indices[label],idx])
		c2size[examplar] += 1
		idx += 1
		
	centerstr = centerstr.rstrip(',')
	#outfile.write("{}\n{}\n{}\n".format(centerstr,N,np.mean(preference)))
	'''
	for i in range(len(labels)):
		betweeness = 0
		if id2ctg[i] in between.keys():
			betweeness = between[id2ctg[i]]
		print(labels[i],id2ctg[i],betweeness)
	'''
if __name__ == "__main__":
	parser = argparse.ArgumentParser()
	
	parser.add_argument('edgefile')
	parser.add_argument('repfile')
	args = parser.parse_args()
	runcluster(args)



from apclusterv.cluster.calscore import *
from apclusterv.cluster.runmcl import *
from apclusterv.newscore.ratiosim import *
from apclusterv.cluster.ctgconnect import *
from apclusterv.cluster.runap import *
from apclusterv.cluster.mergecluster import *

class Arg:
	def __init__(self):
		pass

def maincluster(args):
	#parser = argparse.ArgumentParser()
	#parser.add_argument('map',type=str)
	#parser.add_argument('tab',type=str)
	#parser.add_argument('prot',type=str)
	
	
	#args = parser.parse_args()
	
	
	print("calculate contig-contig alignment and ratio score")
	aggrctgscore(args)
	
	print("generate input for mcl clustering")
	
	gsimargs = Arg()
	setattr(gsimargs, 'map', args.map)
	setattr(gsimargs,'aggr','tmp/ctgscore.csv')
	setattr(gsimargs,'prot',args.prot)
	setattr(gsimargs,'out','tmp/shared_protein.csv')
	
	calglobal(gsimargs)
	
	print("run mcl clustering")
	mclargs = Arg()
	inflation = args.inflation
	setattr(mclargs, 'edges', 'tmp/shared_protein.csv')
	setattr(mclargs,'inflation',inflation)
	run_mcl(mclargs)
	
	
	print("generate input for affinity propagation")
	
	
	apargs = Arg()
	
	setattr(apargs, 'map', args.map)
	setattr(apargs,'alnscore','tmp/ctgscore.csv')
	setattr(apargs,'pcscore','tmp/shared_protein.csv')


	#parser.add_argument("resfile",type=str)
	#parser.add_argument("aggr",type=str)
	#parser.add_argument("map",type=str)
	#filter_edge(apargs)
	#construct_graph(apargs.aggr+".abs",apargs.aggr+".rep")
	
	
	connect(apargs)
	
	print("run affinity propagation")
	runapargs = Arg()
	setattr(runapargs,'edgefile','tmp/shared_protein.csv.aln')
	setattr(runapargs,'repfile','tmp/shared_protein.csv.rep')
	setattr(runapargs,'apk',args.apk)
	runcluster(runapargs)
	
	
	print("merge cluster")
	
	exargs = Arg()
	setattr(exargs,'alnfile','tmp/dvp.ctg.aln.tab')
	setattr(exargs,'apfile','tmp/c2members')
	setattr(exargs,'apscore','tmp/ctgscore.csv')
	setattr(exargs,'map',args.map)
	setattr(exargs,'mcl','tmp/mcl'+str(args.inflation))
	setattr(exargs,'inflation',args.inflation)
	setattr(exargs,'apk',args.apk)

	prepare(exargs.alnfile,exargs.apfile,exargs.apscore)

	print("export result")
	finalres(exargs)
	#prepare(args.alnfile,args.apfile,args.apscore)



from Bio import SeqIO
import subprocess
import argparse
import pandas as pd
import os

def parsefaa(transfile):
	prot = open(transfile,'r')
	data = []
	for record in SeqIO.parse(prot,'fasta'):
		pos = record.id.rfind('_')
		contig = record.id[0:pos]
		data.append([record.id,contig])
	mapdf = pd.DataFrame(data,columns = ['protein_id','contig_id'])

	mapdf['contig'] = mapdf['contig_id'].map(lambda s:s.replace(' ','~'))

	prot2id = dict(zip(mapdf['protein_id'],mapdf.index))
	ctgset = set(mapdf['contig'])
	ctg2id = {}
	for idx,ctg in enumerate(ctgset):
		ctg2id[ctg] = idx
    

	mapdf['ctgid'] = mapdf['contig'].map(lambda x:ctg2id[x])
	prot2ctg = dict(zip(mapdf.index,mapdf['ctgid']))

	mapdf[['protein_id','contig','ctgid']].to_csv('tmp/prot_contig_id.csv',sep=',',index=False)

def parsecsv(csvlist):
	dflist = []
	for csvfile in csvlist:
		df = pd.read_csv(csvfile,sep=',',header=0)
		dflist.append(df)
		
	mapdf = pd.concat(dflist)
	mapdf['contig'] = mapdf['contig_id'].map(lambda s:s.replace(' ','~'))
	prot2id = dict(zip(mapdf['protein_id'],mapdf.index))
	ctgset = set(mapdf['contig'])
	ctg2id = {}
	for idx,ctg in enumerate(ctgset):
		ctg2id[ctg] = idx
    

	mapdf['ctgid'] = mapdf['contig'].map(lambda x:ctg2id[x])
	prot2ctg = dict(zip(mapdf.index,mapdf['ctgid']))

	mapdf[['protein_id','contig','ctgid']].to_csv('tmp/prot_contig_id.csv',sep=',',index=False)	
	
	
	
def main():

	if not os.path.exists("tmp"):
		os.makedirs("tmp")
		
	parser = argparse.ArgumentParser()
	parser.add_argument("contig",help="contig fasta file")
	args = parser.parse_args()
	print("Predict genes (protein sequences) from contig dna sequence")
	#subprocess.run(["prodigal", "-i", args.contig, "-p", "meta" ,"-a", args.contig+".faa", "-o", "genelog"])
	#parsefaa(args.contig+".faa")
	
def fromcsvlist(csvlist):
	parsecsv(csvlist)	
	
if __name__ == "__main__":
	parser = argparse.ArgumentParser()
	#parser.add_argument("contig",help="contig fasta file")
	parser.add_argument("csv1",help="contig fasta file")
	parser.add_argument("csv2",help="db fasta file")
	args = parser.parse_args()
	fromcsvlist([args.csv1,args.csv2])
	
	
	
	
	
	



import pandas as pd
import argparse
import json

#parser = argparse.ArgumentParser()
#parser.add_argument('map',type=str)

#parser.add_argument('exam',type=str)
#parser.add_argument('c2members',type=str)
#parser.add_argument('mcl',type=str)
#args = parser.parse_args()
#id 16192
#a previously unassigned genome (query) is now assigned to the closet genome (target), by aggregating the bitscores of alignments between a protein on the query genome and a protein in the APC set on the target genome. To make such assignment more confident, we require that at least 50% of all the proteins on the query genome can align to the target genome [].  
def assign(ctg1,anno):
	anno = sorted(anno,key=lambda x:x[1],reverse=True)
	answer = anno[0]
	score = answer[1]
	number = answer[2]
	
	return score,number,answer[0]
	
'''
(vcontact2env) yaohaobin@ubuntu:~/share/viral/dvp$ python exec/eval.py newscore/publish_apres.csv fulltax.tab
sensitivity: 6551
108505.0 21346968.0 65075.0 45480.0
0.9948736503541589
0.6599209898811542
(vcontact2env) yaohaobin@ubuntu:~/share/viral/dvp$ python exec/eval.py mcl3 fulltax.tab
sensitivity: 4236
53459.0 8937893.0 26228.0 24298.0
0.9944120015775484
0.6762677859891169
if only a small portion of genomes in a step-2 cluster are supported by a large number of newly-admitted genomes, we will extract the supported minorites from the cluster and let them form a new cluster together with the newly-admitted ones. More specifically, we will split the step-2 genome cluster when supported genomes is less than k=1/4, while the new cluster will be larger than the remaining old cluster. 

'''
def exportres(member2new,member2mcl):
	cl2members = {}
	for member,cl in member2mcl.items():
		cl2members.setdefault(cl,[])
		cl2members[cl].append(member)
	new2members = {}   
	
	
	for member,new in member2new.items():
		new2members.setdefault(new,[])
		new2members[new].append(member)
	
	data = []
	for cl,members in cl2members.items():
		nosupport = []
		supported = []
		
		
		for member in members:
			if member in new2members.keys():
				supported.append(member)
			else:
				nosupport.append(member)
				
		newlist = []
		newlist += supported
		for new in supported:
			newlist += new2members[new]        
				
		print(len(supported),len(nosupport))
		if len(newlist)>len(nosupport) and len(supported)*3 < len(nosupport):
			print("newcluster")
			oldstr = ",".join(nosupport)
			data.append([len(nosupport),oldstr])
			
				 
			print(len(newlist),len(nosupport))
			newstr = ",".join(newlist)
			data.append([len(newlist),newstr])
		else:
			for new in supported:
				 members += new2members[new]
			memberstr = ",".join(members)
			data.append([len(members),memberstr])
	 
	resultdf = pd.DataFrame(data,columns=['Size','Members'])
	resultdf.to_csv('publish_apres.csv')

def finalres(args)
	apfile = open(args.c2members,'r')
	c2members = json.load(apfile)
	ap2members = {}
	member2c = {}
	for c,members in c2members.items():
		ap2members[int(c)] = members
		for member in members:
			member2c[member] = int(c)
		
	mapdf = pd.read_csv(args.map,sep=',',header=0)

	ctg2id = dict(zip(mapdf['contig'],mapdf['ctgid']))
	id2ctg = dict(zip(mapdf['ctgid'],mapdf['contig']))

	ctg2nprot = {}
	for prot,ctg in zip(mapdf['protein_id'],mapdf['ctgid']):
		ctg2nprot.setdefault(ctg,0)
		ctg2nprot[ctg] += 1




	examdf = pd.read_csv(args.exam,header=None,sep=' ',skiprows=1)
	examdf.columns = ['ctg1','ctg2','score','number']

	ctg2anno = {}
	for ctg1,ctg2,score,number in zip(examdf['ctg1'],examdf['ctg2'],examdf['score'],examdf['number']):

	#print(ctg1,ctg2,score,tax1,tax2)

		if ctg1 in ctg2anno.keys():
			ctg2anno[ctg1].append([ctg2,score,number])
		else:
			ctg2anno[ctg1] = [[ctg2,score,number]]

	mcldf = pd.read_csv(args.mcl,sep=',',header=0)
	member2mcl = {}

	member2new = {}
	mclnum = 0

	for idx,row in mcldf.iterrows():
		mclnum += 1
		members = row['Members'].split(',')
		for member in members:
			member2mcl[member.replace(' ','~')] = idx

	ncut = 20  
	p = 0
	n = 0
	np = 0
	nomcl = 0

	pscore = 0
	npscore = 0
	
	admit = 0
	for ctg1 in ctg2anno.keys():
		
		ctgname1 = id2ctg[ctg1]
		if ctgname1 in member2mcl.keys():
			continue
		nprot = ctg2nprot[ctg1]
		if nprot < 2:
			 continue
			
		if ctg1 in member2c.keys():
			continue
		
		
		#ctax = "unsign"
		#if member2c[ctg1] in id2tax.keys():
			#ctax = id2tax[member2c[ctg1]]
		#print(ctg1,tax1)
		score,number,ctg2 = assign(ctg1,ctg2anno[ctg1])
		ctg2name = id2ctg[ctg2]
		ctg1name = id2ctg[ctg1]
		
		if ctg2name in member2mcl.keys():
			if number/nprot >0.5:
	   
				member2new[ctg1name] = ctg2name
				
				
				admit +=1

		 

	
	print(admit)    
	
	#for member,mcl in member2mcl.items():
		#if not member in member2new.keys():
			#member2new[member] = mcl
	exportres(member2new,member2mcl)
	



import numbers
import warnings

import numpy as np

from sklearn.exceptions import ConvergenceWarning
from sklearn.base import BaseEstimator, ClusterMixin
from sklearn.utils import as_float_array, check_random_state
from sklearn.utils import check_scalar
from sklearn.utils.validation import check_is_fitted
from sklearn.metrics import euclidean_distances
from sklearn.metrics import pairwise_distances_argmin
def _equal_similarities_and_preferences(S, preference):
    def all_equal_preferences():
        return np.all(preference == preference.flat[0])

    def all_equal_similarities():
        # Create mask to ignore diagonal of S
        mask = np.ones(S.shape, dtype=bool)
        np.fill_diagonal(mask, 0)

        return np.all(S[mask].flat == S[mask].flat[0])

    return all_equal_preferences() and all_equal_similarities()
def adj_affinity_propagation(
    S,
    *,
    preference=None,
    convergence_iter=15,
    max_d_iter=10,
    max_p_iter = 10,
    pstep = 10,
    max_iter=200,
    damping=0.5,
    copy=True,
    verbose=False,
    return_n_iter=False,
    random_state=None,
):
    """Perform Affinity Propagation Clustering of data.

    Read more in the :ref:`User Guide <affinity_propagation>`.

    Parameters
    ----------

    S : array-like of shape (n_samples, n_samples)
        Matrix of similarities between points.

    preference : array-like of shape (n_samples,) or float, default=None
        Preferences for each point - points with larger values of
        preferences are more likely to be chosen as exemplars. The number of
        exemplars, i.e. of clusters, is influenced by the input preferences
        value. If the preferences are not passed as arguments, they will be
        set to the median of the input similarities (resulting in a moderate
        number of clusters). For a smaller amount of clusters, this can be set
        to the minimum value of the similarities.

    convergence_iter : int, default=15
        Number of iterations with no change in the number
        of estimated clusters that stops the convergence.

    max_iter : int, default=200
        Maximum number of iterations.
    max_d_iter: 
        Maximum super iterations for damping factor steps
    damping : float, default=0.5
        Damping factor between 0.5 and 1.

    copy : bool, default=True
        If copy is False, the affinity matrix is modified inplace by the
        algorithm, for memory efficiency.

    verbose : bool, default=False
        The verbosity level.

    return_n_iter : bool, default=False
        Whether or not to return the number of iterations.

    random_state : int, RandomState instance or None, default=None
        Pseudo-random number generator to control the starting state.
        Use an int for reproducible results across function calls.
        See the :term:`Glossary <random_state>`.

        .. versionadded:: 0.23
            this parameter was previously hardcoded as 0.

    Returns
    -------

    cluster_centers_indices : ndarray of shape (n_clusters,)
        Index of clusters centers.

    labels : ndarray of shape (n_samples,)
        Cluster labels for each point.

    n_iter : int
        Number of iterations run. Returned only if `return_n_iter` is
        set to True.

    Notes
    -----
    For an example, see :ref:`examples/cluster/plot_affinity_propagation.py
    <sphx_glr_auto_examples_cluster_plot_affinity_propagation.py>`.

    When the algorithm does not converge, it will still return a arrays of
    ``cluster_center_indices`` and labels if there are any exemplars/clusters,
    however they may be degenerate and should be used with caution.

    When all training samples have equal similarities and equal preferences,
    the assignment of cluster centers and labels depends on the preference.
    If the preference is smaller than the similarities, a single cluster center
    and label ``0`` for every sample will be returned. Otherwise, every
    training sample becomes its own cluster center and is assigned a unique
    label.

    References
    ----------
    Brendan J. Frey and Delbert Dueck, "Clustering by Passing Messages
    Between Data Points", Science Feb. 2007
    """
    S = as_float_array(S, copy=copy)
    n_samples = S.shape[0]
    
    if S.shape[0] != S.shape[1]:
        raise ValueError("S must be a square array (shape=%s)" % repr(S.shape))

    if preference is None:
        preference = np.median(S)
    
    preference = np.array(preference)

    if n_samples == 1 or _equal_similarities_and_preferences(S, preference):
        # It makes no sense to run the algorithm in this case, so return 1 or
        # n_samples clusters, depending on preferences
        warnings.warn(
            "All samples have mutually equal similarities. "
            "Returning arbitrary cluster center(s)."
        )

        if preference.flat[0] >= S.flat[n_samples - 1]:
            return (
                (np.arange(n_samples), np.arange(n_samples), 0)
                if return_n_iter
                else (np.arange(n_samples), np.arange(n_samples))
            )
        else:
            return (
                (np.array([0]), np.array([0] * n_samples), 0)
                if return_n_iter
                else (np.array([0]), np.array([0] * n_samples))
            )

    random_state = check_random_state(random_state)

    # Place preference on the diagonal of S
  
    #print(S[0,1],S[1,15])
    step_d_iter = max_d_iter
    num_clusters = 0
    unconvergable = False
    optimal = False
    for p_iter in range(max_p_iter):
        print("preference step p_iter %d" % p_iter)
        S.flat[:: (n_samples + 1)] = preference

        A = np.zeros((n_samples, n_samples))
        R = np.zeros((n_samples, n_samples))  # Initialize messages
        # Intermediate results
        tmp = np.zeros((n_samples, n_samples))

        # Remove degeneracies
        S += (
            np.finfo(S.dtype).eps * S + np.finfo(S.dtype).tiny * 100
        ) * random_state.standard_normal(size=(n_samples, n_samples))

        # Execute parallel affinity propagation updates
        e = np.zeros((n_samples, convergence_iter))

        ind = np.arange(n_samples)
        if unconvergable:
            break
        if optimal:
            break
        for d_iter in range(step_d_iter):
        
            if damping > 1:
                unconvergable = True
                break
            for it in range(max_iter):
                # tmp = A + S; compute responsibilities
                np.add(A, S, tmp)
                I = np.argmax(tmp, axis=1)
                Y = tmp[ind, I]  # np.max(A + S, axis=1)
                tmp[ind, I] = -np.inf
                Y2 = np.max(tmp, axis=1)

                # tmp = Rnew
                np.subtract(S, Y[:, None], tmp)
                tmp[ind, I] = S[ind, I] - Y2
                #print(tmp[0,1],tmp[1,15])
                # Damping
                tmp *= 1 - damping
                R *= damping
                R += tmp

                # tmp = Rp; compute availabilities
                np.maximum(R, 0, tmp)
                tmp.flat[:: n_samples + 1] = R.flat[:: n_samples + 1]

                # tmp = -Anew
                tmp -= np.sum(tmp, axis=0)
                dA = np.diag(tmp).copy()
                tmp.clip(0, np.inf, tmp)
                tmp.flat[:: n_samples + 1] = dA
                #print(tmp[0,1],tmp[1,15])
                # Damping
                tmp *= 1 - damping
                A *= damping
                A -= tmp

            # Check for convergence
                E = (np.diag(A) + np.diag(R)) > 0
                e[:, it % convergence_iter] = E
                K = np.sum(E, axis=0)

                if it >= convergence_iter:
                    se = np.sum(e, axis=1)
                    unconverged = np.sum((se == convergence_iter) + (se == 0)) != n_samples
                    if (not unconverged and (K > 0)) or (it == max_iter):
                        never_converged = False
                        if verbose:
                            print("Converged after %d iterations." % damping)
                        break
            else:
                never_converged = True
                if verbose:
                    print("Did not converge")

            I = np.flatnonzero(E)
            K = I.size  # Identify exemplars

            if K > 0:
                if never_converged:
                    warnings.warn(
                        "Affinity propagation d_iter did not converge, this model "
                        "may return degenerate cluster centers and labels.",
                        ConvergenceWarning,
                    )
                    print("daming %f not converged" % damping)
                c = np.argmax(S[:, I], axis=1)
                c[I] = np.arange(K)  # Identify clusters
                # Refine the final set of exemplars and clusters and return results
                for k in range(K):
                    ii = np.where(c == k)[0]
                    j = np.argmax(np.sum(S[ii[:, np.newaxis], ii], axis=0))
                    I[k] = ii[j]

                c = np.argmax(S[:, I], axis=1)
                c[I] = np.arange(K)
                labels = I[c]
                # Reduce labels to a sorted, gapless, list
                cluster_centers_indices = np.unique(labels)
                labels = np.searchsorted(cluster_centers_indices, labels)

                if not never_converged:
                    step_d_iter = 1
                    #print(preference,len(cluster_centers_indices))
                    preference += pstep
                    
                    if num_clusters >= len(cluster_centers_indices):
                        optimal = True
                    num_clusters = len(cluster_centers_indices)
                    break
                damping += 0.1
                if d_iter == max_d_iter - 1:
                    unconvergable = True
            else:
                warnings.warn(
                    "Affinity propagation did not converge and this model "
                    "will not have any cluster centers.",
                    ConvergenceWarning,
                )
                labels = np.array([-1] * n_samples)
                cluster_centers_indices = []
                if d_iter == max_d_iter - 1:
                    unconvergable = True
                damping += 0.1

    if return_n_iter:
        return cluster_centers_indices, labels, it + 1
    else:
        return cluster_centers_indices, labels


import pandas as pd
import numpy as np
import json
import argparse



def filter(examdf,col,supported):
	ctg2anno = {}
	ctg2annonum = {}
	prot2best = {}
	
	#Notice that an ap component is a cluster of proteins. Now we will aggregate protein-protein similarities to the genome-genome level.
	
	outdata = []
	outedgedata = []
	for qprot,tprot,score,ctg1,ctg2 in zip(examdf['qprot'],examdf['tprot'],examdf[col],examdf['ctg1'],examdf['ctg2']):
		if not tprot in supported:
			continue
		
		if (qprot,ctg2) in prot2best.keys():
			continue
		outedgedata.append([qprot,tprot,score,ctg1,ctg2])
		prot2best[(qprot,ctg2) ] = score
		ctg2anno.setdefault(ctg1,{})
		ctg2anno[ctg1].setdefault(ctg2,0)
		
		ctg2annonum.setdefault(ctg1,{})
		ctg2annonum[ctg1].setdefault(ctg2,0)
		
		ctg2anno[ctg1][ctg2] += score
		ctg2annonum[ctg1][ctg2] += 1
		
	for ctg1,anno in ctg2anno.items():
		annonum = ctg2annonum[ctg1]
		for ctg2,score in anno.items():
			
			outdata.append([ctg1,ctg2,score,annonum[ctg2]])

	outedgedf = pd.DataFrame(outedgedata,columns = ['qprot','tprot','score','ctg1','ctg2'])
	outedgedf.to_csv('examedge.csv',index=False)
	outdf = pd.DataFrame(outdata,columns = ['ctg1','ctg2','score','number']) 
	outdf.to_csv('tmp/exam',index=False,sep=',')
		
def pick(alnfile,tset,pairset,t2cut,member2ap):
	df = pd.read_csv(alnfile,sep='\t')
	
	df = df[df['ctg2'].isin(tset)]
	
	#print(len(df))
	whitelist = {}
   
	examlist = []    
	supported =  set()
	for row in df.itertuples():
		if row[7] == row[8]:
			continue
		pairkey = (row[7],row[8])
		if not int(row[8]) in tset:
			continue
		cutoff = t2cut[int(row[8])]
		ratio = int(row[3])*1.0/( int(row[3]) + int(row[4]) + int(row[5]))
		#if row[2] == 101982:
			#print(ratio,cutoff)
			
		#we will include genomes that haven't been assiged to any clusters to the waiting list, as long as they can align to a component center with similarity over its cut-off.
		if not pairkey in pairset:
			if ratio >= cutoff:
			#if True:
				supported.add(row[2])
				examlist.append([row[1],row[2],row[3],row[6],row[7],row[8]])
			continue
		
		
		if ratio >= cutoff:
		#if True:    
			supported.add(row[2])
			#if int(row[6]) == 15046:
				#print(row[7],row[1],row[2],row[3],row[6])
				
			#whitelist:for each ap center, all alignments over the component-wise cut-off will support the center as a confident representative
			if pairkey in whitelist.keys():    
				whitelist[pairkey].append([row[1],row[2],row[3]])
			else:
				whitelist[pairkey] = [[row[1],row[2],row[3]]]
			
	outdata = []
	for pairkey,alns in whitelist.items():
		ctg1,ctg2 = pairkey
		for prot1,prot2,match in alns:
			outdata.append([ctg1,ctg2,prot1,prot2,match,])
			
	outdf = pd.DataFrame(outdata,columns=['query','target','qprot','tprot','match'])
	outdf = outdf.sort_values(by=['qprot','match'],ascending=[True,False])
	outdf.to_csv('whitelist.csv',index=False)        
			
	examdf = pd.DataFrame(examlist,columns = ['qprot','tprot','match','score','ctg1','ctg2'])
	
	filter(examdf,'score',supported)
	
		
def prepare(alnfile,apfile,scorefile):
	
	
	
		
	ap = open(apfile,'r')
	c2members = json.load(ap)           
		   
	scoredf = pd.read_csv(scorefile,sep='\t')
	ap2members = {}
	ap2scores = {}
	member2ap = {}
	for c,members in c2members.items():
		ap2members[int(c)] = members
		ap2scores[int(c)] = []
		for member in members:
			member2ap[member] = int(c)
		
	for ctg1,ctg2,score in zip(scoredf['ctg1'],scoredf['ctg2'],scoredf['ratioscore']):
		if ctg1 == ctg2:
			continue
		if ctg1 in ap2members.keys():
			ap2scores[ctg1].append(score)
		if ctg2 in ap2members.keys():
			ap2scores[ctg2].append(score)
	for ap in ap2scores.keys():
		ap2scores[ap] = np.mean(ap2scores[ap])
		
	#for each ap compoment, the mean values of edges to the representive node is regarded as the cut-off for this component.
	pairset = set()
	for ap,members in ap2members.items():
		for member in members:
			if member == ap:
				continue
			pairkey = (member,ap)
			pairset.add(pairkey)
		
	pick(alnfile,ap2members.keys(),pairset,ap2scores,member2ap)   
			

#parser = argparse.ArgumentParser()
#parser.add_argument('alnfile',type=str)
#parser.add_argument('apfile',type=str)
#parser.add_argument('scorefile',type=str)
#args = parser.parse_args()
#prepare(args.alnfile,args.apfile,args.apscore)




#parser = argparse.ArgumentParser()
#parser.add_argument('map',type=str)

#parser.add_argument('exam',type=str)
#parser.add_argument('c2members',type=str)
#parser.add_argument('mcl',type=str)
#args = parser.parse_args()
#id 16192
#a previously unassigned genome (query) is now assigned to the closet genome (target), by aggregating the bitscores of alignments between a protein on the query genome and a protein in the APC set on the target genome. To make such assignment more confident, we require that at least 50% of all the proteins on the query genome can align to the target genome [].  
def assign(ctg1,anno):
	anno = sorted(anno,key=lambda x:x[1],reverse=True)
	answer = anno[0]
	score = answer[1]
	number = answer[2]
	
	advantage = -1
	second_target = -1 
	if len(anno)>1:
		second = anno[1]
		advantage = score - second[1]
		second_target = second[0]
	return score,number,answer[0],second_target,advantage
	
'''
(vcontact2env) yaohaobin@ubuntu:~/share/viral/dvp$ python exec/eval.py newscore/publish_apres.csv fulltax.tab
sensitivity: 6551
108505.0 21346968.0 65075.0 45480.0
0.9948736503541589
0.6599209898811542
(vcontact2env) yaohaobin@ubuntu:~/share/viral/dvp$ python exec/eval.py mcl3 fulltax.tab
sensitivity: 4236
53459.0 8937893.0 26228.0 24298.0
0.9944120015775484
0.6762677859891169
if only a small portion of genomes in a step-2 cluster are supported by a large number of newly-admitted genomes, we will extract the supported minorites from the cluster and let them form a new cluster together with the newly-admitted ones. More specifically, we will split the step-2 genome cluster when supported genomes is less than k=1/4, while the new cluster will be larger than the remaining old cluster. 

'''
def exportres(member2new,member2mcl,ctg2advantage,admitdf,id2ctg,ctg2id,args):
	cl2members = {}
	for member,cl in member2mcl.items():
		cl2members.setdefault(cl,[])
		cl2members[cl].append(member)
	new2members = {}   
	
	
	for member,new in member2new.items():
		new2members.setdefault(new,[])
		new2members[new].append(member)
	
	data = []

	admitdata = []
	for cl,members in cl2members.items():
		pruneset = judgecluster(ctg2advantage,cl,admitdf,args.apk)
		for ctgid in pruneset:
			ctgname = id2ctg[ctgid]
			admits_ind = new2members[ctgname]
			data.append([len(admits_ind),','.join(admits_ind)])

		nosupport = []
		supported = []
		
		outmembers = []

		for member in members:
			
			if member in new2members.keys():
				ctgid = ctg2id[member]
				if ctgid in pruneset:
					continue
				supported.append(member)
				admitnum = len(new2members[member])
				outmembers.append(member+"#"+str(admitnum))
			else:
				outmembers.append(member+"#0")
				nosupport.append(member)
		memberstr = ",".join(outmembers)
		admitdata.append([cl,memberstr])

		newlist = []
		newlist += supported
		for new in supported:
			newlist += new2members[new]        
				
		#print(len(supported),len(nosupport))
		if len(supported)>=1 and len(supported)*3 < len(nosupport):
		#if len(newlist)>len(nosupport) and len(supported)*3 < len(nosupport):
		#if False:
			#print("newcluster")
			oldstr = ",".join(nosupport)
			data.append([len(nosupport),oldstr])
			
				 
			#print(len(newlist),len(nosupport))
			newstr = ",".join(newlist)
			data.append([len(newlist),newstr])
		else:
			for new in supported:
				 members += new2members[new]
			memberstr = ",".join(members)
			data.append([len(members),memberstr])
	mergedf = pd.DataFrame(admitdata,columns = ['cl','Members'])
	mergedf.to_csv('mergestat.csv',index=False)	 
	resultdf = pd.DataFrame(data,columns=['Size','Members'])
	resultdf.to_csv('tmp/cluster_result.'+str(args.inflation)+'.'+str(args.apk)+'.csv')

def judgecluster(ctg2advantage,querycl,admitdf,apk):
	admit2advantage = {}
	for ctg1,ctg2,cl in zip(admitdf['ctg1'],admitdf['ctg2'],admitdf['cl']):
		if not ctg1 in ctg2advantage.keys():
			continue
		if not ctg2 in ctg2advantage.keys():
			continue
		if querycl != cl:
			continue
		thisadvantage = ctg2advantage[ctg1][2]
		if thisadvantage <0:
			continue
		admit2advantage.setdefault(ctg2,[])
		admit2advantage[ctg2].append(thisadvantage)
		center_best = ctg2advantage[ctg2][0]
		
		#print(ctg1,ctg2,thisadvantage,center_best)
	pruneset = set()
	for admitter in admit2advantage.keys():
		#print(admitter,np.mean(admit2advantage[admitter]),ctg2advantage[admitter])
		center_best = ctg2advantage[admitter][0]
		if center_best in admit2advantage.keys():
			partner_ratio = np.mean(admit2advantage[center_best])
			this_ratio = np.mean(admit2advantage[admitter])
			if partner_ratio + this_ratio and this_ratio<partner_ratio > float(apk):
				#print('prune',admitter,center_best,this_ratio,partner_ratio,querycl)
				pruneset.add(admitter)
	return pruneset
		

def finalres(args):
	apfile = open(args.apfile,'r')
	c2members = json.load(apfile)
	ap2members = {}
	member2c = {}
	for c,members in c2members.items():
		ap2members[int(c)] = members
		for member in members:
			member2c[member] = int(c)
		
	mapdf = pd.read_csv(args.map,sep=',',header=0)

	ctg2id = dict(zip(mapdf['contig'],mapdf['ctgid']))
	id2ctg = dict(zip(mapdf['ctgid'],mapdf['contig']))

	ctg2nprot = {}
	for prot,ctg in zip(mapdf['protein_id'],mapdf['ctgid']):
		ctg2nprot.setdefault(ctg,0)
		ctg2nprot[ctg] += 1




	examdf = pd.read_csv('tmp/exam',header=0,sep=',')
	examdf.columns = ['ctg1','ctg2','score','number']

	ctg2anno = {}
	for ctg1,ctg2,score,number in zip(examdf['ctg1'],examdf['ctg2'],examdf['score'],examdf['number']):

	#print(ctg1,ctg2,score,tax1,tax2)

		if ctg1 in ctg2anno.keys():
			ctg2anno[ctg1].append([ctg2,score,number])
		else:
			ctg2anno[ctg1] = [[ctg2,score,number]]

	mcldf = pd.read_csv(args.mcl,sep=',',header=0)
	member2mcl = {}

	member2new = {}
	mclnum = 0
	admitdata = []

	incluster = []
	for idx,row in mcldf.iterrows():
		mclnum += 1
		members = row['Members'].split(',')
		for member in members:
			member2mcl[member.replace(' ','~')] = idx

	ncut = 20  
	p = 0
	n = 0
	np = 0
	nomcl = 0

	pscore = 0
	npscore = 0
	
	admit = 0
	for ctg1 in ctg2anno.keys():
		
		ctgname1 = id2ctg[ctg1]
		
			
		
		
		
		#ctax = "unsign"
		#if member2c[ctg1] in id2tax.keys():
			#ctax = id2tax[member2c[ctg1]]
		#print(ctg1,tax1)
		score,number,ctg2,second_target,advantage = assign(ctg1,ctg2anno[ctg1])
		incluster.append([ctg1,ctg2,second_target,advantage])



		if ctg1 in member2c.keys():
			continue
		ctg2name = id2ctg[ctg2]
		ctg1name = id2ctg[ctg1]
		if ctgname1 in member2mcl.keys():
			continue
		nprot = ctg2nprot[ctg1]
		if nprot < 2:
			 continue
		if ctg2name in member2mcl.keys():
			if number/nprot >0.5:
				nprot2 = ctg2nprot[ctg2]
				member2new[ctg1name] = ctg2name
				admitdata.append([ctg1,ctg2,member2mcl[ctg2name],number,nprot,nprot2,advantage])
				
				admit +=1

		 

	
	#print(admit)    
	
	#for member,mcl in member2mcl.items():
		#if not member in member2new.keys():
			#member2new[member] = mcl
	
	incluster_df = pd.DataFrame(incluster,columns=['ctg1','ctg2','second','advantage'])
	incluster_df.to_csv('inclusternum.csv',index=False)
	ctg2advantage = {}
	for ctg1,ctg2,second,advantage in zip(incluster_df['ctg1'],incluster_df['ctg2'],incluster_df['second'],incluster_df['advantage']):
		ctg2advantage[ctg1] = [ctg2,second,advantage]

	admitdf = pd.DataFrame(admitdata,columns=['ctg1','ctg2','cl','number','nprot1','nprot2','advantage'])
	admitdf.to_csv('admitpair.csv',index=False)
	exportres(member2new,member2mcl,ctg2advantage,admitdf,id2ctg,ctg2id,args)




import pandas as pd 
import argparse
from apclusterv.utils.protein_mcl import *
from apclusterv.exec.maincluster import *
import pkg_resources
#mcl: mcl result
#mapfile: prot2contigmap
#protmap:  protein cluster map
#protein_id,contig_id,cluster

#def make_protein_clusters_mcl(blast_fp, out_p, inflation=2):



def parsemcl(mcl,mapfile):
	pcidx = 0
	mapdf = pd.read_csv(mapfile,sep=',',header=0)
	prot2ctg = dict(zip(mapdf['protein_id'],mapdf['contig']))
	data = []
	for line in open(mcl,'r'):
		line = line.strip()
		info = line.split('\t')
		
		if len(info)<2:
			continue
		pc = "PC_"+str(pcidx)
		for protein in info:
			#print(protein,prot2ctg[protein],pc)
			data.append([protein,prot2ctg[protein],pc])
		pcidx += 1
		
	df = pd.DataFrame(data,columns=['protein_id','contig_id','cluster'])
	df.to_csv('tmp/protein_cluster_v2.csv',index=False)
	
def parsecsv(csvname):
	mapdf = pd.read_csv(csvname,sep=',',header=0)
	mapdf['contig'] = mapdf['contig_id'].map(lambda s:s.replace(' ','~'))
	prot2id = dict(zip(mapdf['protein_id'],mapdf.index))
	ctgset = set(mapdf['contig'])
	ctg2id = {}
	for idx,ctg in enumerate(ctgset):
		ctg2id[ctg] = idx
    

	mapdf['ctgid'] = mapdf['contig'].map(lambda x:ctg2id[x])
	prot2ctg = dict(zip(mapdf.index,mapdf['ctgid']))

	mapdf[['protein_id','contig','ctgid']].to_csv('tmp/prot_contig_id.csv',sep=',',index=False)		
	
def main():
	if not os.path.exists("tmp"):
		os.makedirs("tmp")
	parser = argparse.ArgumentParser()
	parser.add_argument("-contig",type=str,help='contig dna file for clustering')
	parser.add_argument("-protein",type=str,help='protein sequences translated from the contigs')
	parser.add_argument("-csv",type=str,help="mapping file for proteins and contigs")
	parser.add_argument("-t",type=int,default=1,help="number of threads for alignment, 1 by default")
	parser.add_argument("-evalue",type=float,default=0.0001,help="evalue threshold for protein-protein alignment, 0.0001 by default")
	parser.add_argument("-alnnum",type=int,default=25,help="max target sequence number for protein-protein alignment, 25 by default")
	parser.add_argument("-i",type=float,default=3,help="inflation value for mcl,3 by default" )
	parser.add_argument("-r",type=float,default=4,help="relative simialrity cut-off parameter in the cluster integration step. 4 by default")

	
	args = parser.parse_args()
	if args.protein is not None:
		protfile = args.protein
		if args.csv is None:
			print("you need specify -csv argument if -protein is used")
			exit()
		parsecsv(args.csv)
	else:
		if args.contig is None:
			print("you need specify -contig or -protein")
			exit()
		protfile = args.contig+".faa"
	
	
	print("Running diamond alignment")
	db_fp = make_diamond_db(protfile, 'tmp', args.t)
	run_diamond(protfile, db_fp, args.t, args.evalue, args.alnnum, 'tmp/protein.diamond.tab')
	
	print("Creating protein family")
	
	mclres = make_protein_clusters_mcl('tmp/protein.diamond.tab', 'tmp')

	parsemcl(mclres,"tmp/prot_contig_id.csv")

	clusterargs = Arg()
	setattr(clusterargs,'map',"tmp/prot_contig_id.csv")
	setattr(clusterargs,'tab','tmp/protein.diamond.tab')
	setattr(clusterargs,'prot',"tmp/protein_cluster_v2.csv")
	setattr(clusterargs,'inflation',args.i)
	setattr(clusterargs,'apk',args.r)
	maincluster(clusterargs)


if __name__ == "__main__":
	print("please run with apclusterv")
	main()
	



import pandas as pd

import numpy as np

#parser = argparse.ArgumentParser()
#parser.add_argument('map',type=str)
#parser.add_argument('tab',type=str)
#args = parser.parse_args()

def aggrctgscore(args):
	mapdf = pd.read_csv(args.map,sep=',',header=0)
	prot2id = dict(zip(mapdf['protein_id'],mapdf.index))
	ctg2id = dict(zip(mapdf['contig'],mapdf['ctgid']))
	prot2ctg = dict(zip(mapdf.index,mapdf['ctgid']))

	df = pd.read_csv(args.tab,sep='\t',header=None)
	df = df[[0,1,3,4,5,11]]

	df.columns = ['prot1','prot2','match','mis','gap','alnscore']
	id1series = df['prot1'].map(lambda x:prot2id[x])
	id2series = df['prot2'].map(lambda y:prot2id[y])

	df['prot1'] = id1series
	df['prot2'] = id2series

	df['ctg1'] = df['prot1'].map(lambda x:prot2ctg[x])
	df['ctg2'] = df['prot2'].map(lambda y:prot2ctg[y])
	#print(df.head(100))
	df.to_csv('tmp/dvp.ctg.aln.tab',index=False,sep='\t')


	newdf = df.groupby(['ctg1','ctg2']).agg({'match':[np.sum],'mis':[np.sum],'gap':[np.sum],'alnscore':[np.sum]}).reset_index()
	newdf.columns = ['ctg1','ctg2','match','mis','gap','alnscore']

	key2alnscore = {}
	key2ratioscore = {}
	for row in newdf.itertuples():
		ctg1 = row[1]
		ctg2 = row[2]
		match = row[3]
		mis = row[4]
		gap = row[5]
		alnscore = row[6]		

		ratioscore = match/(match+mis+gap)
		ctga = min(ctg1,ctg2)
		ctgb = max(ctg1,ctg2)
		key = "{}#{}".format(ctga,ctgb)
		if key in key2alnscore.keys():
			key2alnscore[key] = max(key2alnscore[key],alnscore)
			key2ratioscore[key] = max(key2ratioscore[key],ratioscore)

		else:
			key2alnscore[key] = alnscore
			key2ratioscore[key] = ratioscore

	data = []
	for key,alnscore in key2alnscore.items():
		ratioscore = key2ratioscore[key]
		info = key.split('#')
		ctg1 = info[0]
		ctg2 = info[1]
		data.append([ctg1,ctg2,alnscore,ratioscore])

	outdf = pd.DataFrame(data,columns=['ctg1','ctg2','alnscore','ratioscore'])

	outdf.to_csv('tmp/ctgscore.csv',sep='\t',index=False)





#!/usr/bin/env python
#-*- coding:utf-8 -*-
import os 
from setuptools import setup, find_packages

MAJOR =1
MINOR =1
PATCH =2
VERSION = f"{MAJOR}.{MINOR}.{PATCH}"

def get_install_requires():
    reqs = [
            'pandas>=0.18.0',
			'numpy>=1.9.2',
            'networkx>=2.8.4',
            'scipy>=1.8.1',
            'scikit-learn>=1.1.2'
            ]
    return reqs
setup(
	name = "apclusterv",
	version = VERSION,
    author ="haobinherbert",
    author_email = "haobinherbert@163.com",
    long_description_content_type="text/markdown",
	url = 'https://github.com/hbyaoherbert/Apclusterv.git',
	long_description = open('README.md',encoding="utf-8").read(),
    python_requires=">=3.7",
    install_requires=get_install_requires(),
	packages = find_packages(),
    entry_points={
        "console_scripts" : ['apclusterv = apclusterv.cluster_from_prot:main','prepare=apclusterv.prepare_from_contig:main']
    },
 	license = 'Apache',
   	classifiers = [
       'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
       'Natural Language :: English',
       'Operating System :: Unix',
       'Programming Language :: Python',       
       'Programming Language :: Python :: 3.7',
       'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    package_data={'': ['scripts/*.R', '*.txt','.toml']}, #这个很重要
    include_package_data=True #也选上

)



import pandas as pd
from scipy.stats import hypergeom
import argparse
import numpy as np
from multiprocessing import Pool
from timeit import default_timer as timer
import subprocess
import os
import pkg_resources
'''
parser = argparse.ArgumentParser()
parser.add_argument('map',type=str)
parser.add_argument('aggr',type=str)
parser.add_argument('prot',type=str)
args = parser.parse_args()
'''

scorep = {}
def crossctg(pc,ctgset):
	ctglist = list(ctgset)
	N = len(ctglist)
	num = 0
	for i in range(N-1):
		for j in range(i+1,N):
			ctg1 = ctglist[i]
			ctg2 = ctglist[j]
			ctga = min(ctg1,ctg2)
			ctgb = max(ctg1,ctg2)
			ctgkey = (ctga,ctgb)
			if not ctgkey in scorep.keys():
				continue
			num +=1
			scorep[ctgkey] += 1
	#print(pc,num)

def aggrpc(df,ctg2id):
	df = df.dropna()
	df = df.sort_values(by='cluster')
	current_pc = ''
	ctgset = set()
	for prot,ctg,pc in zip(df['protein_id'],df['contig_id'],df['cluster']):
		ctg = ctg.replace(' ','~')
		ctg = ctg2id[ctg]
		if pc == current_pc:
			ctgset.add(ctg)
		else:
			if current_pc !='':
				crossctg(current_pc,ctgset)
			ctgset = set()
			current_pc = pc
	crossctg(current_pc,ctgset)


'''
parser = argparse.ArgumentParser()
parser.add_argument('map',type=str)
parser.add_argument('aggr',type=str)
parser.add_argument('prot',type=str)
args = parser.parse_args()
'''
def calglobal(args):

	mapdf = pd.read_csv(args.map,sep=',',header=0)
	prot2id = dict(zip(mapdf['protein_id'],mapdf.index))
	ctg2id = dict(zip(mapdf['contig'],mapdf['ctgid']))
	id2ctg = dict(zip(mapdf['ctgid'],mapdf['contig']))

	ctg2npc = {}
	df = pd.read_csv(args.prot,sep=',',header=0)
	df = df.sort_values(by='contig_id')
	current_contig = ''
	pcset = set()

	allpc = set()
	for contig,pc in zip(df['contig_id'],df['cluster']):
		allpc.add(pc)
		contig = contig.replace(' ','~')
		contig = ctg2id[contig]
		if contig == current_contig:
			pcset.add(pc)
		else:
			if current_contig != '':
				ctg2npc[current_contig] = len(pcset)
			current_contig = contig
			pcset = set()
			pcset.add(pc)



	ctg2npc[current_contig] = len(pcset)

	aggrdf = pd.read_csv(args.aggr,sep='\t',header=0)
	totalnpc = len(allpc)


	n_ctg = len(ctg2npc)

	T = n_ctg*(n_ctg-1)/2

	for ctg1,ctg2,count in zip(aggrdf['ctg1'],aggrdf['ctg2'],aggrdf['alnscore']):


		ctga = min(ctg1,ctg2)
		ctgb = max(ctg1,ctg2)
		ctgkey = (ctga,ctgb)
		scorep[ctgkey] = 0
	'''
	
	n = min(ctg2npc[ctg1],ctg2npc[ctg2])
	N = max(ctg2npc[ctg1],ctg2npc[ctg2])
	prob = hypergeom.sf(count-1,totalnpc,n,N)
	score = np.nan_to_num(-np.log10(prob) - np.log10(T))
	'''
	#print(ctg1,ctg2,scorep)

#print(len(scorep))



	df = pd.read_csv(args.prot,sep=',',header=0)
	print("start aggr")
	aggrpc(df,ctg2id)
	scores = []
	ctgkeys = []
	ns = []
	Ns = []


	for ctgkey,score in scorep.items():
		if score >= 2:
		
			
			scores.append(score-1)
	
			ctg1,ctg2 = ctgkey
			key = "{}#{}".format(ctg1,ctg2)
			ctgkeys.append(key)
			n = min(ctg2npc[ctg1],ctg2npc[ctg2])
			N = max(ctg2npc[ctg1],ctg2npc[ctg2])
			ns.append(n)
			Ns.append(N)

	outdf = pd.DataFrame()
	outdf["ctgkey"] = ctgkeys
	outdf["k"] = scores
	outdf["n"] = ns
	outdf["N"] = Ns
	outdf.to_csv("tmp/rtest.csv",sep=',',index=False)

	dist = pkg_resources.get_distribution('apclusterv')
 
# 获取包的资源文件路径
	
	rfile = dist.location+'/apclusterv/scripts/phypertest.R'
	subprocess.run(['Rscript', rfile,'tmp/rtest.csv'])
	#Rscript /path/to/your/script.R tmp/rtest.csv
	resdf = pd.read_csv('tmp/phyper.csv',header=0,index_col=0)
	resdf['similarity'] = resdf['res'].map(lambda prob:np.nan_to_num(-np.log10(prob) - np.log10(T)))
	outfile = open(args.out,'w')
	cutoff = 10
	if args.mode == "complete":
		cutoff = 5
	for ctgkey,sim in zip(resdf['ctgkey'],resdf['similarity']):
		if sim < cutoff:
			continue
		info = ctgkey.split('#')
		ctg1 = int(info[0])
		ctg2 = int(info[1])
		ctgname1 = id2ctg[ctg1]
		ctgname2 = id2ctg[ctg2]
		outfile.write(str(ctgname1)+'\t'+str(ctgname2)+'\t'+str(sim)+'\n')
		

	outfile.close()
	



import argparse
import pandas as pd
import numpy as np


#parser = argparse.ArgumentParser()
#parser.add_argument("resfile",type=str)
#parser.add_argument("aggr",type=str)
#parser.add_argument("map",type=str)

#args = parser.parse_args()

#usage: filter_edge(args)
#usage: construct_graph('tmp/'+args.aggr+".abs",'tmp/'+args.aggr+".rep")

ctg2idx = {}
idx2ctg = {}
parent = []
replen = []

def readmcl(filename):
	cl2members = {}
	mclres = pd.read_csv(filename,sep=',',header=0)
	for idx,row in mclres.iterrows():
		cl2members[idx] = row['Members'].split(',')
	return cl2members


def filter_edge(args):

	mapdf = pd.read_csv(args.map,sep=',',header=0)
	prot2id = dict(zip(mapdf['protein_id'],mapdf.index))
	ctg2id = dict(zip(mapdf['contig'],mapdf['ctgid']))
	prot2ctg = dict(zip(mapdf.index,mapdf['ctgid']))


	aggr = pd.read_csv(args.aggr,sep='\t',header=0)

	edge2score = {}

	cl2edges = {}
	ctg2cl = {}
	pabs = []

	rep2members = readmcl(args.resfile)
	for rep,members in rep2members.items():
		for member in members:
			ctg2cl[ctg2id[member]] = rep
	for score,ctg1,ctg2 in zip(aggr['alnscore'],aggr['ctg1'],aggr['ctg2']):
		
		if not ctg1 in ctg2cl.keys():
			pabs.append([ctg1,ctg2,score])
			continue
		if not ctg2 in ctg2cl.keys():
			pabs.append([ctg1,ctg2,score])
			continue

		cl1 = ctg2cl[ctg1]
		cl2 = ctg2cl[ctg2]
		if cl1 == cl2:
			pabs.append([ctg1,ctg2,score])


	pabsdf = pd.DataFrame(pabs,columns = ['node1','node2','score'])

	pabsdf.to_csv(args.aggr+".abs",index=False,sep='\t')




def find_rep(ctgid):
    rep = ctgid
    while parent[rep] != rep:
        rep = parent[rep]
    
    while parent[ctgid] != rep:
        nextnode = parent[ctgid]
        parent[ctgid] = rep
        ctgid = nextnode
    
    return rep

def find_rep_merge(ctgid,rep_after_merge):
    

    rep = ctgid
    while parent[rep] != rep:
        rep = parent[rep]
    if rep == rep_after_merge:
        return rep

    if replen[rep] + replen[rep_after_merge] > 200:
        return rep
    while parent[ctgid] != rep:
        nextnode = parent[ctgid]
        parent[ctgid] = rep_after_merge
        ctgid = nextnode

    replen[rep_after_merge] += replen[rep]
    parent[rep] = rep_after_merge

    return rep

def construct(complement_e):
    edges = []
    edge1 = pd.read_csv(complement_e,sep='\t',header=0)
    


    for ctg1,ctg2,score in zip(edge1['node1'],edge1['node2'],edge1['score']):

        if not ctg1 in ctg2idx.keys():
            thisid = len(ctg2idx)
            ctg2idx[ctg1] = thisid
            idx2ctg[thisid] = ctg1
            parent.append(thisid)
            replen.append(1)
        if not ctg2 in ctg2idx.keys():
            thisid = len(ctg2idx)
            ctg2idx[ctg2] = thisid
            idx2ctg[thisid] = ctg2 
            parent.append(thisid)
            replen.append(1)
        edges.append([ctg1,ctg2,score])
    edges = sorted(edges,key=lambda d:d[2],reverse=True)
    
   
    return edges
'''
def readlen(filename):
    ctg2len = {}
    for line in open(filename,'r'):
        line = line.strip()
        info = line.split('\t')
        ctg2len[info[0]] = int(info[1])
    return ctg2len
    

'''
def cluster(edges):
    for ctg1,ctg2,score in edges:
        
        if score < 0.3:
            continue

        ctgid1 = ctg2idx[ctg1]
        ctgid2 = ctg2idx[ctg2]

        longctg = max(ctgid1,ctgid2)
        shortctg = min(ctgid1,ctgid2)

        #if aln/shortlen < 0.75 or pid <90:
            #continue

        
        replong =  find_rep(longctg)
        repshort = find_rep_merge(shortctg,replong)
        #print(shortctg,longctg,repshort,replong,shortlen,longlen,aln,pid)

def printcluster(outname):
    outfile = open(outname,'w')
    for i in range(len(parent)):
        rep = parent[i]
        while rep!=parent[rep]:
            rep = parent[rep]
        
        outfile.write('\t'.join([str(i),str(rep),str(idx2ctg[i])])+'\n')
            
            
def construct_graph(edgedata,clusterout):
#parser = argparse.ArgumentParser()
#parser.add_argument('complement_e')
#parser.add_argument('complement_rep')
#args = parser.parse_args()

	edges = construct(edgedata)
	cluster(edges)
	printcluster(clusterout) 
	



import argparse
import pandas as pd
import json

#parser = argparse.ArgumentParser()
#parser.add_argument("alnscore",type=str)
#parser.add_argument("pcscore",type=str)
#parser.add_argument("map",type=str)

#args = parser.parse_args()
def bfs(graph, nodes):
	
	seen = set()
	result = []   #List of Lists to hold the final result 
	for node in nodes:
		if node not in seen:
			components = set()
			leaves = [node]
			while leaves:
				leaf = leaves.pop()
				seen.add(leaf)
				components.add(leaf)
				for connected_node in graph[leaf]: 
					if connected_node not in seen: leaves.append(connected_node)
			print(len(components))
			result.append(components) 
	return result
def connect(args):
	alndf = pd.read_csv(args.alnscore,sep='\t',header=0)
	edge2aln = {}
	for ctg1,ctg2,score in zip(alndf['ctg1'],alndf['ctg2'],alndf['ratioscore']):
		ctga = min(ctg1,ctg2)
		ctgb = max(ctg1,ctg2)
		edge2aln[(ctga,ctgb)] = score 
	

	mapdf = pd.read_csv(args.map,sep=',',header=0)
	prot2id = dict(zip(mapdf['protein_id'],mapdf.index))
	ctg2id = dict(zip(mapdf['contig'],mapdf['ctgid']))
	prot2ctg = dict(zip(mapdf.index,mapdf['ctgid']))



	df = pd.read_csv(args.pcscore,header=None,sep='\t')
	ctgset = set()
	graph = {}
	outscore = []
	for idx,row in df.iterrows():
		ctg1 = row[0]
		ctg2 = row[1]
	
		if ctg1==ctg2:
			continue
		ctg1 = ctg2id[ctg1]
		ctg2 = ctg2id[ctg2]

		ctga = min(ctg1,ctg2)
		ctgb = max(ctg1,ctg2)
		key = (ctga,ctgb)
		if key in edge2aln.keys():
			alnscore = edge2aln[key]
			outscore.append([ctga,ctgb,alnscore])


		ctgset.add(ctg1)
		ctgset.add(ctg2)
		if not ctg1 in graph.keys():
			graph[ctg1] = []
		if not ctg2 in graph.keys():
			graph[ctg2] = []
		graph[ctg1].append(ctg2)
		graph[ctg2].append(ctg1)

	scoredf = pd.DataFrame(outscore,columns=['ctg1','ctg2','ratioscore'])
	scoredf.to_csv(args.pcscore+'.aln',index=False,sep=',')
	results = bfs(graph,ctgset)
	print(len(graph))
	print(len(results))
	outdata = []
	idx = 0
	for components in results:
		for ctg in components:
			outdata.append([idx,ctg])
		idx += 1
	outdf = pd.DataFrame(outdata,columns=['rep','contig'])
	outdf.to_csv(args.pcscore+'.rep',sep=',')
